/************************************************************************
 *   IRC - Internet Relay Chat, common/common_def.h
 *   Copyright (C) 2005 Rusnet
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *   $Id$
 */
#ifndef _CONVERSION_H_
#define _CONVERSION_H_

#include <iconv.h>

/* ircd conversion descriptor
 * don't use any fields directly but via functions below, please */
typedef struct conversion
{
  struct conversion *next;
  struct conversion *prev;
  int inuse;
  char *charset; /* allocated */
  iconv_t cdin; /* charset --> internal, NULL if charset==internal */
  iconv_t cdout; /* internal --> charset */
} conversion_t;

/* for internal use only, see macros below */
size_t conv_do_conv(iconv_t, const unsigned char *, size_t, unsigned char **, size_t);

/* public functions */
conversion_t *conv_get_conversion(const char *);
void conv_free_conversion(conversion_t *);
conversion_t *conv_set_internal(conversion_t **old, const char *);
#define conv_inherit(parent,new) new=parent, parent->inuse++
#define conv_charset(conv) conv->charset
#define conv_do_in(conv,in,insize,out,outsize) conv_do_conv(conv->cdin,in,insize,out,outsize)
#define conv_do_out(conv,in,insize,out,outsize) conv_do_conv(conv->cdout,in,insize,out,outsize)

/* helpers for transcoding fields to new internal codepage from old one (conv)
 * note: new codepage must be different from current!
 * note2: buff size for conv_transcode() must not be less of field size! */
#define conv_transcode(conv,field,buff) do { unsigned char *c=buff;\
					    size_t sz=conv_do_conv(conv->cdin,field,strlen(field),&c,sizeof(field)-1);\
					    memcpy(field,buff,sz);\
					    field[sz]=0;} while(0)
#define conv_transcode_realloc(conv,field,buff) do { unsigned char *c=buff;\
						    size_t sz=conv_do_conv(conv->cdin,field,strlen(field),&c,sizeof(buff));\
						    MyFree(field);\
						    field=MyMalloc(sz+1);\
						    memcpy(field,buff,sz);\
						    field[sz]=0;} while(0)

void conv_report(aClient *, char *);

size_t rfcstrtoupper(char *dst, char *src, size_t);

#endif
